<?php

namespace Tygh\Addons\SwBifitOnline;

use Tygh\Registry;

/**
 * Logs
 *
 * @package Tygh\Addons\SwBifitOnline
 */
class Logs
{
    protected $timeLog;
    
    protected $db;

    /**
     * Logs constructor.
     */
    public function __construct()
    {
        $this->timeLog = !empty(Registry::get('addons.sw_bifit_online.time_log')) ? Registry::get('addons.sw_bifit_online.time_log') * 86400 : 90 * 86400;
    }

    /**
     * Searches logs.
     *
     * @param array|null    $conditions List of the search conditions
     * @param int           $limit      Limit
     * @param int           $offset     Offset
     *
     * @return array
     */
    public function search(array $conditions = array(), $limit = 20, $offset = 0)
    {
        $sql_condition = '1 = 1';

        if ($conditions) {
            $sql_condition = db_quote('?w', $conditions);
        }

        $rows = db_get_array(
            'SELECT * FROM ?:bifit_online_logs WHERE ?p ORDER BY id DESC LIMIT ?i OFFSET ?i',
            $sql_condition, $limit, $offset
        );

        return $rows;
    }

    /**
     * Logs the request.
     *
     * @param string        $url            URL
     * @param string        $request        json request data
     * @param string        $response       json response data
     * @param int           $status         Status true or false
     * @param null|string   $message        Message
     *
     * @return int Log identifier of the inserted data.
     */
    public function log($url, $request, $response, $message = '')
    {
        $this->deleteOld();
        
        $is_error = $status = false;
        
        if(!empty($response)){
            if(strpos($response,'502 Bad Gateway')){
                $is_error = true;
                $message .= ' Server Error 502 ';
            }
            
            if(strpos($response,'error')){
                $is_error = true;
            }
            
            $res = json_decode($response, true);
            
            if(isset($res['type']) && $res['type'] == 'ERROR'){
                $is_error = true;
                $message .= ' '.$res['message'].' ';
            }
            
            if(isset($res['error'])){
                $is_error = true;
                $message .= ' '.$res['status'].': '.$res['message'];
            }
        }
        
        if($is_error == false){
            $status = true;
        }

        $data = array(
            'timestamp' => time(),
            'url' => $url,
            'request' => $request,
            'response' => $response,
            'message' => $message,
            'status' => $status
        );
        
        return db_query('INSERT INTO ?:bifit_online_logs ?e', $data);
    }
    
    /**
    * Deletes old logs.
    */
    public function deleteOld()
    {
        $timestamp = time() - $this->timeLog;
        db_query('DELETE FROM ?:bifit_online_logs WHERE timestamp <= ?i', $timestamp);
    }
    
    /**
     * Gets count logs by conditions.
     *
     * @return int
     */
    public function count(array $conditions = array())
    {
        $sql_condition = '1 = 1';

        if ($conditions) {
            $sql_condition = db_quote('?w', $conditions);
        }

        $cnt = db_get_field(
            'SELECT COUNT(*) FROM ?:bifit_online_logs WHERE ?p',
            $sql_condition
        );

        return $cnt;
    }
}